/***************************************************************************
                           clogfile.cpp  -  description
                             -------------------
    begin                : Sat Dec 14 2002
    copyright            : (C) 2002-2005 by Mathias Kster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>
#include <stdio.h>

#ifndef WIN32
#include <stdarg.h>
#endif

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cbytearray.h>
#include "clogfile.h"

/** */
static CThread LogfileThread;

/** */
CLogFile::CLogFile()
{
}

/** */
CLogFile::~CLogFile()
{
}

/** */
bool CLogFile::Write( CString file, eLogType type, char * message, ... )
{
	va_list args;
	CByteArray msg_buff;
	int i;

	LogfileThread.Lock();

	msg_buff.SetSize(32768);

#ifdef WIN32
	va_start( args, message );
	i = _vsnprintf((char*)msg_buff.Data(), 32768, message, args);
#else
	va_start( args, message );
	i = vsnprintf((char*)msg_buff.Data(), 32768, message, args);
#endif

	va_end(args);

	if ( i > 32767 )
	{
		msg_buff.SetSize(0);
		i = 0;
	}

	LogfileThread.UnLock();

	return Write( file, type, CString().Set((char*)msg_buff.Data(),i) );
}

/** */
bool CLogFile::Write( CString file, eLogType /*type*/, CString message )
{
	bool res = FALSE;
	char timestamp[23];
	struct tm * t;
	time_t ti;
	FILE * logfile;
	CString newline;
	
	LogfileThread.Lock();

	ti = time(0);
	t  = localtime(&ti);
	
	strftime(timestamp, sizeof(timestamp), "[%F.%H:%M:%S] ", t);
	
	// TODO: fix logfiles and add option to set logpath ...
/*
	if ( CConfig::Instance() )
	{
		file = CConfig:Instance()->GetLogPath() + file;
	}
*/
	if ( (logfile = fopen(file.Data(),"a")) != 0 )
	{
		fwrite(timestamp,22,1,logfile);
		fwrite(message.Data(),message.Length(),1,logfile);
		newline = "\n";
		fwrite(newline.Data(),newline.Length(),1,logfile);
		fclose(logfile);
		res = TRUE;
	}

	LogfileThread.UnLock();

	return res;
}
